@@ -2,26 +2,89 @@ import pika, requests, threading, json, time
 from logzero import logger
 from pymitter import EventEmitter
 from autosolveclient.autosolveconstants import AutoSolveConstants
+from autosolveclient.autosolve_errors import AutoSolveConnectionError
+
+
+class Instance:
+    auto_solve = None
 
 
 class AutoSolve:
 
+    def __init__(self, params):
+        self.params = params
+
+    def get_instance(self):
+        if Instance.auto_solve is None:
+            Instance.auto_solve = InternalAutoSolve(self.params)
+            return Instance.auto_solve
+        else:
+            return Instance.auto_solve
+
+
+class InternalAutoSolve:
+
     def __init__(self, params):
         self.autosolve_constants = AutoSolveConstants()
         self.emitter = EventEmitter()
+        self.vhost = self.autosolve_constants.VHOST
 
         self.access_token = params["access_token"]
         self.api_key = params["api_key"]
         self.client_key = params["client_key"]
         self.debug = params["debug"]
         self.should_alert_on_cancel = params["should_alert_on_cancel"]
-        self.manual_reconnect = params["manual_reconnect"]
 
+        self.routing_key = ""
+        self.send_routing_key = ""
+        self.account_id = ""
+        self.directExchangeName = ""
+        self.fanoutExchangeName = ""
+        self.receiver_queue_name = ""
+        self.routing_key_receiver = ""
+        self.routing_key_cancel = ""
+        self.token_send_routing_key = ""
+        self.cancel_send_routing_key = ""
+
+        self.delay = None
+        self.attempts = None
+        self.connection = None
+        self.channel = None
+        self.cancel_channel = None
+        self.consumer_thread = None
+        self.ready = False
+        self.connected = False
+
+        self.message_backlog = []
+
+    def initialized(self):
+        while self.attempts < self.autosolve_constants.MAX_ATTEMPTS:
+            time.sleep(.5)
+            if self.ready:
+                break
+        return self.ready
+
+    def init(self, access_token=None, api_key=None):
+        self.attempts = 0
+        self.delay = self.autosolve_constants.RETRY_DELAY
+        self.connection = None
+        self.channel = None
+        self.cancel_channel = None
+        self.consumer_thread = None
+        self.ready = False
+        self.connected = False
+
+        if access_token is not None and api_key is not None:
+            self.reset_credentials(access_token, api_key)
+
+        self.set_routing_objects()
+        self.begin_connection()
+
+    def set_routing_objects(self):
         self.routing_key = self.api_key.replace("-", "")
         self.send_routing_key = self.access_token.replace("-", "")
 
         self.account_id = self.access_token.split("-")[0]
-        self.vhost = self.autosolve_constants.VHOST
 
         self.directExchangeName = self.build_with_account_id(self.autosolve_constants.DIRECT_EXCHANGE)
         self.fanoutExchangeName = self.build_with_account_id(self.autosolve_constants.FANOUT_EXCHANGE)
@@ -34,50 +97,45 @@ class AutoSolve:
         self.token_send_routing_key = self.build_with_access_token(self.autosolve_constants.TOKEN_SEND_ROUTE)
         self.cancel_send_routing_key = self.build_with_access_token(self.autosolve_constants.CANCEL_SEND_ROUTE)
 
-        self.connection = None
-        self.channel = None
-        self.cancel_channel = None
-        self.consumer_thread = None
-        self.ready = False
-        self.connected = False
-
-        self.message_backlog = []
-
-    def initialized(self):
-        while True:
-            time.sleep(.5)
-            if self.ready:
-                break
-        return True
-
-    def init(self):
-        if self.manual_reconnect is None:
-            self.manual_reconnect = False
-        if self.check_input_values():
-            self.begin_connection()
+    def reset_credentials(self, access_token, api_key):
+        self.close_connection()
+        self.access_token = access_token
+        self.api_key = api_key
 
     def begin_connection(self):
-        self.consumer_thread = threading.Thread(target=self.connect)
-        valid = self.validate_connection()
-        if valid == 400:
-            self.log(self.autosolve_constants.INVALID_CLIENT_KEY, self.autosolve_constants.ERROR)
-            raise AuthException(self.autosolve_constants.INVALID_CLIENT_KEY)
-        if valid == 401:
-            self.log(self.autosolve_constants.INVALID_API_KEY_OR_ACCESS_TOKEN, self.autosolve_constants.ERROR)
-            raise AuthException(self.autosolve_constants.INVALID_API_KEY_OR_ACCESS_TOKEN)
+        if self.check_input_values():
+            self.close_connection()
+            status_code = self.validate_connection()
+            if status_code == 200:
+                self.log("Validation complete", self.autosolve_constants.SUCCESS)
+                self.consumer_thread = threading.Thread(target=self.connect)
+                self.consumer_thread.start()
+            elif status_code == 400:
+                self.log(self.autosolve_constants.INVALID_CLIENT_KEY, self.autosolve_constants.ERROR)
+                raise AuthException(self.autosolve_constants.INVALID_CLIENT_KEY)
+            elif status_code == 429:
+                self.log(self.autosolve_constants.TOO_MANY_REQUESTS, self.autosolve_constants.ERROR)
+                raise AuthException(self.autosolve_constants.TOO_MANY_REQUESTS)
+            else:
+                self.log(self.autosolve_constants.INVALID_API_KEY_OR_ACCESS_TOKEN, self.autosolve_constants.ERROR)
+                raise AuthException(self.autosolve_constants.INVALID_API_KEY_OR_ACCESS_TOKEN)
         else:
-            self.log("Validation complete", self.autosolve_constants.SUCCESS)
-            self.consumer_thread.start()
+            raise AuthException(self.autosolve_constants.INVALID_API_KEY_OR_ACCESS_TOKEN)
 
     def connect(self):
         self.log("Beginning connection establishment", self.autosolve_constants.STATUS)
+        sleep_time = self.get_delay(self.delay)
+        self.delay += 1
+        if sleep_time > 0:
+            time.sleep(sleep_time)
 
         credentials = pika.PlainCredentials(self.account_id, self.access_token)
         parameters = pika.ConnectionParameters(host=self.autosolve_constants.HOSTNAME,
                                                port=5672,
                                                virtual_host=self.vhost,
                                                credentials=credentials,
-                                               connection_attempts=50,
+                                               connection_attempts=10,
+                                               retry_delay=5,
                                                blocked_connection_timeout=60)
         try:
             self.connected = self.establish_connection(parameters) \
@@ -85,23 +143,33 @@ class AutoSolve:
                              and self.register_response_queue()
 
             if self.connected:
+
                 self.log("Beginning message consumption", self.autosolve_constants.SUCCESS)
                 self.ready = True
+                self.delay = 0
                 self.channel.start_consuming()
+                self.log("Stopped Message Consuming", self.autosolve_constants.SUCCESS)
+                self.emitter.emit("AutoSolveError", AutoSolveConnectionError.CONNECTION_CLOSED_UNEXPECTEDLY)
             else:
                 self.log("Error with RabbitMQ connection, attempting to re-establish", self.autosolve_constants.WARNING)
+                self.handle_connection_error()
 
         except pika.exceptions.AMQPConnectionError or pika.exceptions.ConnectionClosedByBroker:
             self.log("Error with RabbitMQ connection, attempting to re-establish", self.autosolve_constants.WARNING)
-            self.emitter.emit("AutoSolveError", self.autosolve_constants.CONNECTION_ERROR_INIT)
+            self.emitter.emit("AutoSolveError", AutoSolveConnectionError.CONNECTION_ERROR_INIT)
             self.handle_connection_error()
+
         except Exception:
-            self.log("Unknown error occured during connection. Aborting", self.autosolve_constants.ERROR)
-            self.emitter.emit("AutoSolveError", self.autosolve_constants.CONNECTION_ERROR_INIT)
+            self.log("Unknown error occurred during connection.", self.autosolve_constants.ERROR)
+            self.emitter.emit("AutoSolveError", AutoSolveConnectionError.CONNECTION_ERROR_INIT)
+            self.handle_connection_error()
 
     def close_connection(self):
-        if self.connection is not None:
+        self.log("Closing Connection", self.autosolve_constants.STATUS)
+        if self.connection is not None and self.connection.is_open:
             self.connection.close()
+        else:
+            self.log("Connection does not exist", self.autosolve_constants.STATUS)
 
     def establish_connection(self, parameters):
         self.log("Attempting connection establishment", self.autosolve_constants.STATUS)
@@ -137,7 +205,7 @@ class AutoSolve:
                                        on_message_callback=self.receive_message)
             return True
         except pika.exceptions.ChannelClosedByBroker:
-            self.emitter.emit("AutoSolveError", self.autosolve_constants.CONNECTION_ERROR_INIT)
+            self.emitter.emit("AutoSolveError", AutoSolveConnectionError.CONNECTION_ERROR_INIT)
 
     def receive_message(self, channel, method_frame, header_frame, body):
         if method_frame.routing_key == self.routing_key_receiver:
@@ -157,23 +225,20 @@ class AutoSolve:
 
     def handle_connection_error(self):
         self.connected = False
-        if (self.manual_reconnect):
-            self.emitter.emit("AutoSolveError", self.autosolve_constants.CONNECTION_LOST_ERROR)
-        else:
-            while True:
-                try:
-                    self.connect()
-                except pika.exceptions.ConnectionClosedByBroker:
-                    break
-                except pika.exceptions.AMQPConnectionError:
-                    continue
+        self.attempts += 1
 
-            if self.connected:
-                self.log("Connection attempt successful, sending any unsent messages", self.autosolve_constants.SUCCESS)
-                self.attempt_message_backlog_send()
-            else:
-                self.log("Connection attempts unsuccessful", self.autosolve_constants.ERROR)
-                self.emitter.emit("AutoSolveError", self.autosolve_constants.CONNECTION_REESTABLISH_FAILED)
+        self.log("Handling reconnect", self.autosolve_constants.STATUS)
+        try:
+            self.begin_connection()
+        except Exception as ex:
+            print(ex)
+
+        if self.connected:
+            self.log("Connection attempt successful, sending any unsent messages", self.autosolve_constants.SUCCESS)
+            self.attempt_message_backlog_send()
+        else:
+            self.log("Connection attempts unsuccessful", self.autosolve_constants.ERROR)
+            self.emitter.emit("AutoSolveError", AutoSolveConnectionError.CONNECTION_REESTABLISH_FAILED)
 
     def send(self, message, route, exchange):
         message["createdAt"] = round(time.time())
@@ -189,7 +254,7 @@ class AutoSolve:
                                           body=byte_string)
 
     def send_token_request(self, message):
-        self.log("Sending request message for task: " + message['taskId'], self.autosolve_constants.STATUS)
+        self.log("Sending request message for task: " + str(message['taskId']), self.autosolve_constants.STATUS)
         try:
             send_result = self.send(message, self.token_send_routing_key,
                                     self.directExchangeName)
@@ -292,12 +357,18 @@ class AutoSolve:
     def build_with_account_id(self, prefix):
         return prefix + "." + self.account_id
 
+    def get_delay(self, n):
+        if n < len(self.autosolve_constants.SEQUENCE):
+            return self.autosolve_constants.SEQUENCE[n]
+        else:
+            return self.autosolve_constants.SEQUENCE[len(self.autosolve_constants.SEQUENCE) - 1]
+
     def log(self, message, type):
-        if type == self.autosolve_constants.ERROR:
-            logger.error(message)
-        if type == self.autosolve_constants.WARNING:
-            logger.warning(message)
         if self.debug:
+            if type == self.autosolve_constants.ERROR:
+                logger.error(message)
+            if type == self.autosolve_constants.WARNING:
+                logger.warning(message)
             if type == self.autosolve_constants.STATUS:
                 logger.debug(message)
             if type == self.autosolve_constants.SUCCESS:
@@ -318,3 +389,5 @@ class InputValueException(Exception):
 
     def __str__(self):
         return 'InputValueException: {}'.format(self.value)
+
+
